package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Customer;
import org.zheng.demo.data.Order;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

class JacksonTest {
	private Customer cust = new Customer("Zheng", 30);
	private ObjectMapper ob = new ObjectMapper();
	private Order order = new Order();

	@Test
	void test_annotation_for_nonNull() {
		cust.setEmail("test@test.com");

		String jsonStr;
		try {
			jsonStr = ob.writerWithDefaultPrettyPrinter().writeValueAsString(cust);

			assertFalse(jsonStr.contains("null"),
					"Json string contains non-null due to JsonInclude.Include.NON_NULL set at class level.");

			System.out.println(jsonStr);

			Customer person2 = ob.readValue(jsonStr, Customer.class);

			assertEquals("Zheng", person2.getName());
			assertEquals(30, person2.getId());
			assertEquals("test@test.com", person2.getEmail());

		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_include_null_as_default() {	 
		String jsonStr;
		try {

			jsonStr = ob.writeValueAsString(order);
			assertTrue(jsonStr.contains("null"), "Json string contains null by Jackson default");
			assertFalse(jsonStr.contains("custTag"),
					"set JsonInclude.Include.NON_NULL on the custTag, so won't include");
			// include null element
			System.out.println(jsonStr); // {"productName":null,"quantity":0}

			Order data = ob.readValue(jsonStr, Order.class);

			assertNull(data.getOrderTag());

		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_include_null_as_default_with_exclusion() { 
		order.setOrderTag("serialize if not null");
		String jsonStr;
		try {

			jsonStr = ob.writeValueAsString(order);
			assertTrue(jsonStr.contains("null"), "Json string contains null by Jackson default");
			assertTrue(jsonStr.contains("orderTag"),
					"set JsonInclude.Include.NON_NULL on the orderTag. include as it has value.");

			// include null element
			System.out.println(jsonStr);
			// {"orderTag":"serialize if not null","productName":null,"quantity":0}

			Order data = ob.readValue(jsonStr, Order.class);

			assertEquals("serialize if not null", data.getOrderTag());

		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_mixed_annotation() {
		cust.addOrder(order);
		String jsonStr;
		try {
			jsonStr = ob.writerWithDefaultPrettyPrinter().writeValueAsString(cust);

			assertFalse(jsonStr.contains("\"email\" : null"),
					"Json string contains non-null due to JsonInclude.Include.NON_NULL set at Customer class level.");
			assertTrue(jsonStr.contains("\"productName\" : null"),
					"Json string contains non-null due to JsonInclude.Include.NON_NULL set at Customer class level.");

			System.out.println(jsonStr);

			Customer person2 = ob.readValue(jsonStr, Customer.class);

			assertEquals("Zheng", person2.getName());
			assertEquals(30, person2.getId());
	
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_setSerializationInclusion_for_nonNull() {
		String jsonStr;
		try {
			ob.setSerializationInclusion(Include.NON_NULL);
			jsonStr = ob.writeValueAsString(order);
			assertFalse(jsonStr.contains("null"),
					"Json string contains non-null when setSerializationInclusion() is configured globally");

			// only include non-null element
			System.out.println(jsonStr);// {"quantity":0}

			Order data = ob.readValue(jsonStr, Order.class);
			assertNull(data.getOrderTag());

		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

}
